import Core_ws from "./websocket/index";
import { startPublish, startPlay, closeMine, closeOhter } from "./webrtc/index";
import { getCatalog, getCallType } from "./utils/index";
import media from "./media/index";
import ajax from "./api/index";
// import {
//   sysCallHangup,
//   sysCallMonitor,
//   getOrgStructure,
//   getContacts,
//   meetList,
//   tempMeet,
//   getMeetMembers,
//   exportMeetDetail,
//   meetKick,
//   meetAdd,
//   meetOff,
//   meetOut,
//   meetDetail,
// } from "./api/index";

class Core {
  constructor() {
    this.ws = null;
    this.userInfo = null;
    this.media = media;
    this.ajax = ajax;
    // this.ajax = {
    //   sysCallHangup,
    //   sysCallMonitor,
    //   getOrgStructure,
    //   getContacts,
    //   meetList,
    //   tempMeet,
    //   getMeetMembers,
    //   exportMeetDetail,
    //   meetKick,
    //   meetAdd,
    //   meetOff,
    //   meetOut,
    //   meetDetail,
    // };
  }

  //开始 登陆成功发送心跳   ws连接失败/登陆失败都会reject
  login({ url, account, password }) {
    return new Promise(async (resolve, reject) => {
      try {
        this.ws = new Core_ws({ url, account, password, token: 123456 });
        await this.ws.createWebsocket();
        this.ws
          .login()
          .then((res) => {
            this.userInfo = res.data;
            this.ws.heart();
            resolve(res);
          })
          .catch((e) => {
            reject(e);
          });
      } catch (error) {
        reject(error);
      }
    });
  }
  //退出
  logout() {
    this.ws.logout();
  }

  //推流
  publish({
    pcInfo,
    url,
    target,
    type,
    mediaConstraints = { audio: true, video: true, type: "p2p" },
    identifier = "",
  }) {
    /* u:uuid 
      c:类型1语音/2音视频/3视频/4语音对讲/5视频兑奖/6语音会议/视频会议 
      f:自己
      t:目标
      s:签名
      i:用户自定义 传什么返回什么
      */
    if (!(url && target)) {
      console.error("必填参数url、target不能为空");
      return;
    }
    if (!mediaConstraints.type) {
      //type默认点对点
      mediaConstraints.type = "p2p";
    }
    let u = new Date().getTime() + "_" + this.userInfo.user.account;
    let f = this.userInfo.user.account;
    let s = 123456;
    let c = getCatalog(mediaConstraints);
    let t = target;
    let i = JSON.stringify(identifier);
    //真正的推流地址

    if (url.lastIndexOf("/") === url.length - 1) {
      url = url.substring(0, url.lastIndexOf("/"));
    }
    let newUrl = `${url}/${f}${t}_${f}?u=${u}&c=${c}&f=${f}&t=${t}&s=${s}&i=${i}`;
    console.log("newUrl ===>", newUrl);
    let obj = {
      pcInfo,
      url: newUrl,
      type,
      mediaConstraints: {
        audio: mediaConstraints.audio,
        video: mediaConstraints.video,
      },
      indefinite: {},
    };
    return startPublish(obj);
  }

  //播流
  player({
    pcInfo,
    url,
    target,
    call_type,
    rewrite_url = true,
    identifier = "",
  }) {
    if (!(url && target)) {
      console.error("必填参数url、target不能为空");
      return;
    }
    let u = new Date().getTime() + "_" + this.userInfo.user.account;
    let f = this.userInfo.user.account;
    let s = 123456;
    let c = getCallType(call_type);
    let t = target;
    let i = JSON.stringify(identifier);
    //真正的推流地址
    let newUrl;
    if (rewrite_url) {
      url = url.substring(0, url.lastIndexOf("/"));
      newUrl = `${url}/${t}${f}_${t}?u=${u}&c=${c}&f=${f}&t=${t}&s=${s}&i=${i}`;
    } else {
      newUrl = `${url}?u=${u}&c=${c}&f=${f}&t=${t}&s=${s}`;
    }

    let obj = {
      pcInfo,
      url: newUrl,
    };
    return startPlay(obj);
  }

  //停止推流  pcInfo:调用publish/player后返回的对象  user_id:监听者的用户id  uuid:通话的uuid
  closeVideo({ pcInfo, user_id, uuid }) {
    let obj;
    if (pcInfo && pcInfo.close instanceof Function) {
      obj = closeMine(pcInfo);
    } else if (user_id && uuid) {
      obj = closeOhter({ user_id, uuid });
    } else {
      obj = null;
    }
    return obj;
  }
}

export default Core;
