"use strict";
import $ from "jquery";

function SrsRtcPublisherAsync() {
  var self = {};
  try {
    self.publish = async function ({
      url,
      type = "userMedia",
      mediaConstraints = { audio: true, video: true },
    }) {
      var conf = self.__internal.prepareUrl(url);
      self.pc.addTransceiver("audio", { direction: "sendonly" });
      self.pc.addTransceiver("video", { direction: "sendonly" });

      var stream = {};
      if (type === "userMedia") {
        stream = await navigator.mediaDevices.getUserMedia({
          audio: mediaConstraints.audio,
          video: mediaConstraints.video,
        });
      } else if (type === "displayMedia") {
        stream = await navigator.mediaDevices.getDisplayMedia({
          audio: true,
          video: { width: 640, height: 320 },
        });
      }

      stream.getTracks().forEach(function (track) {
        self.pc.addTrack(track);
      });

      var offer = await self.pc.createOffer();
      await self.pc.setLocalDescription(offer);
      var session = await new Promise(function (resolve, reject) {
        var data = {
          api: conf.apiUrl,
          tid: conf.tid,
          streamurl: conf.streamUrl,
          clientip: null,
          sdp: offer.sdp,
        };
        $.ajax({
          type: "POST",
          url: conf.apiUrl,
          data: JSON.stringify(data),
          contentType: "application/json",
          dataType: "json",
        })
          .done(function (data) {
            if (data.code) {
              reject(data);
              return;
            }
            resolve(data);
          })
          .fail(function (reason) {
            reject(reason);
          });
      });
      await self.pc.setRemoteDescription(
        new RTCSessionDescription({ type: "answer", sdp: session.sdp })
      );
      session.simulator =
        conf.schema +
        "//" +
        conf.urlObject.server +
        ":" +
        conf.port +
        "/rtc/v1/nack/";

      self.onaddstream && self.onaddstream({ stream: stream });

      return session;
    };
  } catch (error) {}

  self.close = function () {
    self.pc && self.pc.close();
    self.pc = null;
  };

  self.onaddstream = function (event) {};

  self.__internal = {
    defaultPath: "/rtc/v1/publish/",
    prepareUrl: function (webrtcUrl) {
      var urlObject = self.__internal.parse(webrtcUrl);

      var schema = urlObject.user_query.schema;
      schema = schema ? schema + ":" : window.location.protocol;

      var port = urlObject.port || 8099;
      if (schema === "https:") {
        port = urlObject.port || 443;
      }

      var api = urlObject.user_query.play || self.__internal.defaultPath;
      if (api.lastIndexOf("/") !== api.length - 1) {
        api += "/";
      }
      apiUrl =
        process.env.NODE_ENV === "production"
          ? window.origin + api
          : "https://192.168.10.23:8000" + api;
      for (var key in urlObject.user_query) {
        if (key !== "api" && key !== "play") {
          apiUrl += "&" + key + "=" + urlObject.user_query[key];
        }
      }
      var apiUrl = apiUrl.replace(api + "&", api + "?");

      var streamUrl = urlObject.url;

      return {
        apiUrl: apiUrl,
        streamUrl: streamUrl,
        schema: schema,
        urlObject: urlObject,
        port: port,
        tid: Number(parseInt(new Date().getTime() * Math.random() * 100))
          .toString(16)
          .substr(0, 7),
      };
    },
    parse: function (url) {
      var a = document.createElement("a");
      a.href = url
        .replace("rtmp://", "https://")
        .replace("webrtc://", "https://")
        .replace("rtc://", "https://");

      var vhost = a.hostname;
      var app = a.pathname.substr(1, a.pathname.lastIndexOf("/") - 1);
      var stream = a.pathname.substr(a.pathname.lastIndexOf("/") + 1);

      app = app.replace("...vhost...", "?vhost=");
      if (app.indexOf("?") >= 0) {
        var params = app.substr(app.indexOf("?"));
        app = app.substr(0, app.indexOf("?"));

        if (params.indexOf("vhost=") > 0) {
          vhost = params.substr(params.indexOf("vhost=") + "vhost=".length);
          if (vhost.indexOf("&") > 0) {
            vhost = vhost.substr(0, vhost.indexOf("&"));
          }
        }
      }

      if (a.hostname === vhost) {
        var re = /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/;
        if (re.test(a.hostname)) {
          vhost = "__defaultVhost__";
        }
      }
      var schema = "rtmp";
      if (url.indexOf("://") > 0) {
        schema = url.substr(0, url.indexOf("://"));
      }

      var port = a.port;
      if (!port) {
        if (schema === "http") {
          port = 80;
        } else if (schema === "https") {
          port = 443;
        } else if (schema === "rtmp") {
          port = 1935;
        }
      }

      var ret = {
        url: url,
        schema: schema,
        server: a.hostname,
        port: port,
        vhost: vhost,
        app: app,
        stream: stream,
      };
      self.__internal.fill_query(a.search, ret);

      if (!ret.port) {
        if (schema === "webrtc" || schema === "rtc") {
          if (ret.user_query.schema === "https") {
            ret.port = 443;
          } else if (window.location.href.indexOf("https://") === 0) {
            ret.port = 443;
          } else {
            ret.port = 8099;
          }
        }
      }

      return ret;
    },
    fill_query: function (query_string, obj) {
      obj.user_query = {};

      if (query_string.length === 0) {
        return;
      }

      if (query_string.indexOf("?") >= 0) {
        query_string = query_string.split("?")[1];
      }

      var queries = query_string.split("&");
      for (var i = 0; i < queries.length; i++) {
        var elem = queries[i];

        var query = elem.split("=");
        obj[query[0]] = query[1];
        obj.user_query[query[0]] = query[1];
      }

      if (obj.domain) {
        obj.vhost = obj.domain;
      }
    },
  };

  self.pc = new RTCPeerConnection(null);

  return self;
}

function SrsRtcPlayerAsync() {
  var self = {};
  self.play = async function (url) {
    var conf = self.__internal.prepareUrl(url);
    self.pc.addTransceiver("audio", { direction: "recvonly" });
    self.pc.addTransceiver("video", { direction: "recvonly" });

    var offer = await self.pc.createOffer();
    await self.pc.setLocalDescription(offer);
    var session = await new Promise(function (resolve, reject) {
      var data = {
        api: conf.apiUrl,
        tid: conf.tid,
        streamurl: conf.streamUrl,
        clientip: null,
        sdp: offer.sdp,
      };

      $.ajax({
        type: "POST",
        url: conf.apiUrl,
        data: JSON.stringify(data),
        contentType: "application/json",
        dataType: "json",
      })
        .done(function (data) {
          if (data.code) {
            reject(data);
            return;
          }

          resolve(data);
        })
        .fail(function (reason) {
          reject(reason);
        });
    });
    await self.pc.setRemoteDescription(
      new RTCSessionDescription({ type: "answer", sdp: session.sdp })
    );
    return session;
  };

  self.close = function () {
    self.pc && self.pc.close();
    self.pc = null;
  };

  self.onaddstream = function (event) {};

  self.__internal = {
    defaultPath: "/rtc/v1/play/",
    prepareUrl: function (webrtcUrl) {
      console.log("webrtcUrl ===>", webrtcUrl);
      var urlObject = self.__internal.parse(webrtcUrl);

      var schema = urlObject.user_query.schema;
      schema = schema ? schema + ":" : window.location.protocol;

      var port = urlObject.port || 8099;
      if (schema === "https:") {
        port = urlObject.port || 443;
      }

      var api = urlObject.user_query.play || self.__internal.defaultPath;
      if (api.lastIndexOf("/") !== api.length - 1) {
        api += "/";
      }
      apiUrl =
        process.env.NODE_ENV === "production"
          ? window.origin + api
          : "https://192.168.10.23:8000" + api;
      for (var key in urlObject.user_query) {
        if (key !== "api" && key !== "play") {
          apiUrl += "&" + key + "=" + urlObject.user_query[key];
        }
      }
      var apiUrl = apiUrl.replace(api + "&", api + "?");

      var streamUrl = urlObject.url;

      return {
        apiUrl: apiUrl,
        streamUrl: streamUrl,
        schema: schema,
        urlObject: urlObject,
        port: port,
        tid: Number(parseInt(new Date().getTime() * Math.random() * 100))
          .toString(16)
          .substr(0, 7),
      };
    },
    parse: function (url) {
      var a = document.createElement("a");
      a.href = url
        .replace("rtmp://", "https://")
        .replace("webrtc://", "https://")
        .replace("rtc://", "https://");

      var vhost = a.hostname;
      var app = a.pathname.substr(1, a.pathname.lastIndexOf("/") - 1);
      var stream = a.pathname.substr(a.pathname.lastIndexOf("/") + 1);

      app = app.replace("...vhost...", "?vhost=");
      if (app.indexOf("?") >= 0) {
        var params = app.substr(app.indexOf("?"));
        app = app.substr(0, app.indexOf("?"));

        if (params.indexOf("vhost=") > 0) {
          vhost = params.substr(params.indexOf("vhost=") + "vhost=".length);
          if (vhost.indexOf("&") > 0) {
            vhost = vhost.substr(0, vhost.indexOf("&"));
          }
        }
      }

      if (a.hostname === vhost) {
        var re = /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/;
        if (re.test(a.hostname)) {
          vhost = "__defaultVhost__";
        }
      }

      var schema = "rtmp";
      if (url.indexOf("://") > 0) {
        schema = url.substr(0, url.indexOf("://"));
      }

      var port = a.port;
      if (!port) {
        if (schema === "http") {
          port = 80;
        } else if (schema === "https") {
          port = 443;
        } else if (schema === "rtmp") {
          port = 1935;
        }
      }

      var ret = {
        url: url,
        schema: schema,
        server: a.hostname,
        port: port,
        vhost: vhost,
        app: app,
        stream: stream,
      };
      self.__internal.fill_query(a.search, ret);

      if (!ret.port) {
        if (schema === "webrtc" || schema === "rtc") {
          if (ret.user_query.schema === "https") {
            ret.port = 443;
          } else if (window.location.href.indexOf("https://") === 0) {
            ret.port = 443;
          } else {
            ret.port = 8099;
          }
        }
      }

      return ret;
    },
    fill_query: function (query_string, obj) {
      obj.user_query = {};

      if (query_string.length === 0) {
        return;
      }

      if (query_string.indexOf("?") >= 0) {
        query_string = query_string.split("?")[1];
      }

      var queries = query_string.split("&");
      for (var i = 0; i < queries.length; i++) {
        var elem = queries[i];

        var query = elem.split("=");
        obj[query[0]] = query[1];
        obj.user_query[query[0]] = query[1];
      }

      if (obj.domain) {
        obj.vhost = obj.domain;
      }
    },
  };

  self.pc = new RTCPeerConnection(null);
  self.pc.onaddstream = function (event) {
    if (self.onaddstream) {
      self.onaddstream(event);
    }
  };

  return self;
}

function SrsRtcFormatSenders(senders, kind) {
  var codecs = [];
  senders.forEach(function (sender) {
    sender.getParameters().codecs.forEach(function (c) {
      if (kind && sender.track.kind !== kind) {
        return;
      }

      if (
        c.mimeType.indexOf("/red") > 0 ||
        c.mimeType.indexOf("/rtx") > 0 ||
        c.mimeType.indexOf("/fec") > 0
      ) {
        return;
      }

      var s = "";

      s += c.mimeType.replace("audio/", "").replace("video/", "");
      s += ", " + c.clockRate + "HZ";
      if (sender.track.kind === "audio") {
        s += ", channels: " + c.channels;
      }
      s += ", pt: " + c.payloadType;

      codecs.push(s);
    });
  });
  return codecs.join(", ");
}
export { SrsRtcPublisherAsync, SrsRtcPlayerAsync, SrsRtcFormatSenders };
