import { SrsRtcPublisherAsync, SrsRtcPlayerAsync } from "../lib/rtc.base";
// import { sysCallHangup, sysCallMonitor } from "../api/index";
import ajax from "../api/index";

//推流
function startPublish({
  pcInfo, //pcInfo peerconnectionInfo  连接信息
  url,
  type = "userMedia",
  mediaConstraints = { audio: true, video: true },
  indefinite,
}) {
  return new Promise((resolve, reject) => {
    if (pcInfo) {
      pcInfo.close();
    }
    pcInfo = new SrsRtcPublisherAsync();
    pcInfo.pc.onicegatheringstatechange = function (event) {
      console.log("onicegatheringstatechange   ===>", event);
      if (pcInfo.pc.iceGatheringState === "complete") {
        console.log("ok");
      }
    };

    //推流地址
    pcInfo
      .publish({ url, type, mediaConstraints })
      .then(function (session) {
        console.log("session ===>", session);
      })
      .catch(function (reason) {
        console.error(reason);
        reject(reason);
        pcInfo.close();
      });

    //自定义回显数据  传啥返回啥
    pcInfo.indefinite = indefinite;

    //推流成功后获取本地流
    pcInfo.onaddstream = (stream) => {
      //获取流之后在播流
      let streamAction = streamActionTo(stream, pcInfo);
      Object.assign(pcInfo, stream, streamAction);
      resolve(pcInfo);
    };
  });
}

//播流
function startPlay({ pcInfo, url }) {
  return new Promise((resolve, reject) => {
    if (pcInfo) {
      pcInfo.close();
    }
    url = url.replace("rtmp://", "webrtc://");
    pcInfo = new SrsRtcPlayerAsync();
    pcInfo
      .play(url)
      .then(function (session) {
        console.log(session);
      })
      .catch(function (reason) {
        pcInfo.close();
        reject(reason);
        console.error(reason);
      });

    //推流成功后获取本地流
    pcInfo.onaddstream = (obj) => {
      //获取流之后在播流
      let streamAction = streamActionTo(obj.stream, pcInfo);
      Object.assign(pcInfo, { stream: obj.stream }, streamAction);
      resolve(pcInfo);
    };
  });
}

//流添加动作
function streamActionTo(stream, pcInfo) {
  let action = {
    //禁言
    mute: () => {
      stream.getAudioTracks()[0] &&
        (stream.getAudioTracks()[0].enabled = false);
    },
    //取消禁言
    unmute: () => {
      stream.getAudioTracks()[0] && (stream.getAudioTracks()[0].enabled = true);
    },
    //关闭视频传输
    closeVideo: () => {
      stream.getVideoTracks()[0] &&
        (stream.getVideoTracks()[0].enabled = false);
    },
    //打开视频传输
    openVideo: () => {
      stream.getVideoTracks()[0] && (stream.getVideoTracks()[0].enabled = true);
    },
    closeStream: () => {
      if (pcInfo) {
        pcInfo.close();
        pcInfo.stream.getTracks().forEach((item) => {
          item.stop();
        });
      }
    },
  };
  return action;
}

//关闭自己流
function closeMine(pcInfo) {
  return new Promise((resolve, reject) => {
    try {
      pcInfo.close();
      pcInfo.stream &&
        pcInfo.stream.getTracks().forEach((item) => {
          item.stop();
        });
      resolve();
    } catch (error) {
      reject();
    }
  });
}

//http 关闭他人流
function closeOhter({ user_id, uuid }) {
  return ajax.sysCallHangup({ user_id, uuid });
}

export { startPublish, startPlay, closeMine, closeOhter };
