import noticeProcess from "./noticeProcess";

export default class Core_ws {
  //url:地址 account:账号 password:密码 clientIP:唯一参数
  constructor({ url, account, password, clientIP }) {
    if (!(url && account && password)) {
      console.error("websocket创建失败,信息缺失");
      return null;
    }
    this.url = url;
    this.account = account;
    this.password = password;
    this.clientIP = clientIP;
    this.heartTimer = null;
    this.websock = null;
    this._message = {}; //消息中心
    this.promisePool = {}; //承诺池
  }
  createWebsocket() {
    return new Promise((resolve, reject) => {
      this.websock = new WebSocket(this.url);

      this.websock.onopen = (e) => {
        console.log("WebSocket连接成功");
        resolve({ e, ws: this });
      };

      this.websock.onerror = (e) => {
        this.stopHeart();
        console.log("WebSocket连接发生错误", e);
        reject("WebSocket连接发生错误");
      };

      this.websock.onmessage = (e) => {
        // try {
        let data = JSON.parse(e.data);
        let isNotice = data.cmd !== "rsp_msg"; //cmd 是通知   req_cmd是请求返回 判断req===rsp_msg
        let cmd = isNotice ? data.cmd : data.req_cmd;
        if (cmd === "heart") return;
        if (isNotice) {
          noticeProcess[cmd] &&
            noticeProcess[cmd]({ cmd, data, callBack: this.fire() }); //内部消息处理 callBack用户回调
        } else {
          const { msgid, res_code } = data;
          const req = this.promisePool[msgid];
          if (!req) return;
          res_code === "0" ? req.resolve(data) : req.reject(data);
          delete this.promisePool[msgid];
        }
        // }
        // catch (error) {
        //   console.error(`message异常:${error}`);
        // }
      };

      this.websock.onclose = (e) => {
        this.stopHeart();
        console.log("websocketOnclose ===>", e);
      };
    });
  }

  webSend(data) {
    // 数据发送
    this.websock.send(JSON.stringify(data));
  }

  //事件注册
  on(type, fn) {
    if (!this._message[type]) {
      this._message[type] = [fn];
    } else {
      this._message[type].push(fn);
    }
    return this;
  }
  //事件触发  闭包存住当前this 不然接收消息在noticeProcess回调时this是undefined
  fire() {
    let _this = this;
    return function (type, data) {
      if (!_this._message[type]) return;
      _this._message[type].forEach((fn) => {
        fn(data);
      });
    };
  }

  // 内部操作
  login() {
    return new Promise((resolve, reject) => {
      const data = {
        cmd: "login",
        data: {
          account: this.account,
          password: this.password,
          token: this.token,
          type: "web",
        },
        msgid: new Date().getTime() + this.account,
      };
      this.promisePool[data.msgid] = {
        resolve,
        reject,
      };
      this.webSend(data);
    });
  }

  logout() {
    let data = { cmd: "logout", msgid: new Date().getTime() };
    this.webSend(data);
    this.websock = null;
  }
  heart() {
    this.heartTimer = setInterval(() => {
      const data = {
        cmd: "heart",
        data: {
          lat: "0",
          lng: "0",
          direction: "0",
          battery: "0",
          speed: "0",
          map_type: 1,
        },
        msgid: new Date().getTime(),
      };
      this.webSend(data);
    }, 10000);
  }
  stopHeart() {
    this.heartTimer && window.clearInterval(this.heartTimer);
    this.heartTimer = null;
  }
}
